/*									CircBuff.h     definition of TcircularBuffer.
	Data is written to the end of the buffer and read from the start, when
	either the read or write pointer reaches the end of the buffer it moves
	around to the start again, as one would expect.
	The names of most member functions should be self-explanatory: linearize
	shuffles the data so that the read pointer is at the start of the physical
	buffer and returns the address of the read pointer.
*/

#ifndef _CIRCBUFF_H
#define _CIRCBUFF_H

class TcircularBuffer
	{unsigned int data_waiting;
	unsigned int buffer_size;
	char *wp,*rp;
	char *e_of_buf;
	char *buffer;
	void bump(char **ptrToPtr){*ptrToPtr=(*ptrToPtr==e_of_buf)?buffer:++(*ptrToPtr);}
	unsigned int read(char *dest,unsigned int size,int discard);
public:
	TcircularBuffer(unsigned int size=1024);
	~TcircularBuffer();
	inline unsigned int add(char data);
	unsigned int add(char *string){while(*string)add(*(string++));return data_waiting;}
	unsigned int peek(char *dest,unsigned int size){return read(dest,size,0);}
	unsigned int read(char *dest,unsigned int size){return read(dest,size,1);}
	unsigned int getAvailable(){return data_waiting;}
	inline unsigned int discard(unsigned int size);
	char *linearize();
	char *getHead(){return rp;} // NO GUARANTEE of linearity !!!!!
	inline void clearAll();
	};

#endif  //_CIRCBUFF_H