/*========================================================================
		* Compile this program with Test Stack Overflow OFF.
		  WORD ALIGNMENT MUST BE OFF
		  or casting to byte-aligned structures and unions doesn't work !
  *------------------------------------------------------------------*/
#define _RS232_A_CPP // so our #include file knows its us (for an extern)
#include <dos.h>
#include "RS232_A.h"  // second version of my OO comms handler

// 09/08/94 added monitor buffer.
// 16/08/94 removed transmission buffer

/*static char *Diagnostic[]=
	{{"No errors"},
	{"Time out waiting for CTS"},
	{"Time out waiting for RTS"},
	{"Assignment of null port"},
	{"Attempt to select zero baud rate"},
	{"Attempt to use zero port base address"},
	{"Attempt to set bad bit-length"},
	{"Attempt to set illegal number of stop-bits"},
	{"Unrecognized parity"}
	};
*/
//:::::::::::::::::::::::::::::: The ISR :::::::::::::::::::::::::::::::::::

void interrupt com_int(...) // Handle communications interrupts for TheRS232
	{disable();
	TheRS232->handleComInt();
	enable();
	}

//::::::::::::::::::: Constructor & destructor :::::::::::::::::::::::::::::

RS232::RS232(int port,int speed,int parity,int bits,int stop_bit,TcircularBuffer *Rx,TcircularBuffer *Mon)
	{Rx_buf=Rx;
	Monitor_buf=Mon;
	error=good;
	clearEchoOn();
	clear_cr_translate();
	portbase=0;
	isActive=0;
	InterruptsOn=0;
	if(!SetPort(port)&&!SetSpeed(speed))
			SetPar_bits_stop(parity,bits,stop_bit);
	}

RS232::~RS232()
	{
	deActivate();
	//resvects(); // should have been done... but funny interrupts sometimes
	}           // crop up after programme termination!

//:::::::::::::::::::::Port set-up functions ::::::::::::::::::::::::::::::::

int RS232::SetPort(int Port) /* Set the port number to use */
	{int Offset,far*RS232_Addr;
	selectedPort=Port;         // new, NMH
	switch(Port)               /*Sorts out the base address*/
		{case COM1:
			Offset=0x0000;
			break;
		case COM2:
			Offset=0x0002;
			break;
		default:
			return-1;
		}
	RS232_Addr=(int far*)MK_FP(0x0040,Offset); /*Find out where the port is.*/
	if(*RS232_Addr==0)                     /*If NULL then port not used.*/
		{error=null_port;
		return(-1);
		}
	portbase=*RS232_Addr;                     /* Otherwise set port base*/
	return(0);
	}

/* This routine sets the speed; will accept funny baud rates. */
/* Setting the speed requires that the DLAB be set on.        */

int RS232::SetSpeed(int Speed)
	{if(Speed==0)/*Avoid divide by zero*/
		{error=zero_speed;
		return(-1);
		}
	int divisor=(int)(115200L/Speed);
	if(portbase==0)
		{error=zero_pbase;
		return(-1);
		}
	disable();
	char c=inportb(portbase+LCR);
	outportb(portbase+LCR,(c|0x80));        /*Set DLAB*/
	outportb(portbase+DLL,(divisor&0x00FF));
	outportb(portbase+DLH,((divisor>>8)&0x00FF));
	outportb(portbase+LCR,c);/*Reset DLAB*/
	enable();
	return(0);
	}

int RS232::SetPar_bits_stop(int Parity,int Bits,int StopBit)/* Set other communications parameters */
	{if(portbase==0)
		{error=zero_pbase;
		return(-1);
		}
	if(Bits<5||Bits>8)
		{error=bad_bitlen;
		return(-1);
		}
	if(StopBit!=1&&StopBit!=2)
		{error=bad_stopbits;
		return(-1);
		}
	if(Parity!=none&&Parity!=odd&&Parity!=even)
		{error=bad_parity;
		return(-1);
		}
	int setting=Bits-5;
	setting|=((StopBit==1)?0x00:0x04);
	setting|=Parity;
	disable();
	outportb(portbase+LCR,setting);
	enable();
	return 0;
	}

// :::::::::::::::::Start up and enable functions::::::::::::::::::::::::::

int RS232::Activate()
	{if(!isActive)
		{if(error==good)
			{setvects();
			comm_on();
			if(error==good)
				isActive=1;
			else
				{comm_off();   // switched order 18/08/94 ... should help.
				resvects();
				}
			}
		}
	return error;
	}

void RS232::setvects() //Install our handlers
	{oldvect=getvect((selectedPort==COM1)?0x0C:0x0B);
	setvect((selectedPort==COM1)?0x0C:0x0B,com_int);
	/* we used to do this ...
	oldvects[0]=getvect(0x0B);  // com2
	oldvects[1]=getvect(0x0C);  // com1
	setvect(0x0B,com_int);
	setvect(0x0C,com_int);    */
	}

void RS232::comm_on() // Tell modem that we're ready to go
	{//int pnum=(portbase==COM1BASE?COM1:COM2);
	//intsOn(pnum);
	intsOn();
	int c=inportb(portbase+MCR)|DTR|RTS;
	outportb(portbase+MCR,c);
	}

void RS232::intsOn(/*int pnum*/)     // Turn on communications interrupts
	{if(InterruptsOn)
		return;
	disable();
	int c=inportb(portbase+MCR)|MC_INT;
	outportb(portbase+MCR,c);
	outportb(portbase+IER,RX_INT);
	/* at this point the code used to read ...
	c=inportb(IMR)&(pnum==COM1?IRQ4:IRQ3);
	outportb(IMR,c);
	... which seemed to deny the mouse (com1) any interrupts although com2
	was working. I have changed this, and it seems satisfactory. */
	//oldIMR=inportb(IMR);
	c=inportb(IMR)&(IRQ3&IRQ4);
	outportb(IMR,c);
	enable();
	InterruptsOn=1;
	}

// :::::::::::::::::Shut down and disable functions::::::::::::::::::::::::::

int RS232::deActivate()
	{if(isActive)
		{isActive=0;
		comm_off();   // switched order 18/08/94 ... should help.
		resvects();
		}
	return error;
	}

void RS232::comm_off() //Go off-line
	{intsOff();
	outportb(portbase+MCR,0);
	}

void RS232::intsOff(void) // Turn off communications interrupts
	{if(!InterruptsOn)
		return;
	disable();
	//int c;
	int c=inportb(IMR)|~IRQ3|~IRQ4;
	outportb(IMR,c);
	//outportb(IMR,oldIMR);
	outportb(portbase+IER,0);
	c=inportb(portbase+MCR)&~MC_INT;
	outportb(portbase+MCR,c);
	InterruptsOn=0;
	enable();
	}

void RS232::resvects() //Uninstall our vectors before shut-down
	{setvect((selectedPort==COM1)?0x0C:0x0B,oldvect);
	/* used to be like this ...
	setvect(0x0B,oldvects[0]);
	setvect(0x0C,oldvects[1]); */
	}

// :::::::::::::::::Input function, called by ISR ::::::::::::::::::::::::::

void RS232::handleComInt()
	{if((inportb(portbase+IIR)&RX_MASK)==RX_ID)
		{char c=inportb(TheRS232->portbase+RXR);
		Rx_buf->add(c);
		if(Monitor_buf)
			Monitor_buf->add(c);
		if(echo_mode)
			--echo_expected;
		}
	outportb(ICR,EOI);	/*Signal end of hardware interrupt*/
	}

// ::::::::::::::::: Output functions ::::::::::::::::::::::::::::::::::::

int RS232::out(char data)  //Output a character to the serial port
	{long int timeout=0x0000FFFFL;
	outportb(portbase+MCR,MC_INT|DTR|RTS);
	while((inportb(portbase+MSR)&CTS)==0) //Wait for Clear To Send from modem
		if(!(--timeout))
			{error=cts_timeout;
			return-1;
			}
	timeout = 0x0000FFFFL;
	while((inportb(portbase+LSR)&XMTRDY)==0)  /* Wait for transmitter to clear */
		if(!(--timeout))
			{error=rts_timeout;
			return-1;
			}
	disable();
	if(echo_mode)
		{echo_expected=1;
		if(Txcr_to_Rxcrnl&&data==0x0D)
			++echo_expected;
		}
	outportb(portbase+TXR,data);
	enable();
	if(echo_mode)
		while(echo_expected>0)
			;
	return 0;
	}

int RS232::out(char *data)  //Output a string to the serial port
	{while(*data)
		{int result=out(*data);
		if(!result)
			data++;
		else
			return result;
		}
	return 0;
	}

int RS232::out(TcircularBuffer *buf)    // flush a buffer to the serial port
	{char data;
	while(buf->read(&data,1))
		{if(out(data)==-1)
			return -1;
		if(Monitor_buf)
			Monitor_buf->add(data);
		}
	return 0;
	}

